/******************************************************************************
 GMTextLink.cc

	Implements algorithms required to make portions of the text act
	like buttons.

	Derived classes must implement:

		GetLinkCount
			Returns the number of links in the text.

		GetLinkRange
			Returns the character range of the specified link.

		LinkClicked
			Take whatever action is appropriate for the specified link.

	BASE CLASS = JXTEBase

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include <GMTextLink.h>

#include <JXDisplay.h>
#include <JXHelpManager.h>

#include <JTextEditor.h>
#include <JRegex.h>
#include <JString.h>
#include <JColormap.h>

#include <jXGlobals.h>

#include <jAssert.h>

const JCharacter* regexStr  = "(http://|ftp://|mailto:)[^][:space:]\'`;:()<>[\"]+";
const JCharacter* regexStr2 = "[^][:space:]\'`;:<>[\"]+@[^][:space:]\'`;:()<>[\"]+";
const JCharacter* regexStr3 = "www\\.[^][:space:]\'`;:()<>[\"]+";

/******************************************************************************
 Constructor

 ******************************************************************************/

GMTextLink::GMTextLink
	(
	JXDisplay* 		display,
	JTextEditor* 	editor
	)
	:
	JXTextLink(display),
	JBroadcaster(),
	itsEditor(editor)
{
	itsLinks = new JArray<LinkInfo>;
	assert(itsLinks != NULL);

	itsRegex = new JRegex();
	assert(itsRegex != NULL);

	itsRegex2 = new JRegex();
	assert(itsRegex2 != NULL);

	itsRegex3 = new JRegex();
	assert(itsRegex3 != NULL);

	JError err = itsRegex->SetPattern(regexStr);
	assert(err.OK());

	err = itsRegex2->SetPattern(regexStr2);
	assert(err.OK());

	err = itsRegex3->SetPattern(regexStr3);
	assert(err.OK());
	
	ListenTo(itsEditor);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

GMTextLink::~GMTextLink()
{
	const JSize linkCount = itsLinks->GetElementCount();
	for (JSize i = 1; i <= linkCount; i++)
		{
		LinkInfo info = itsLinks->GetElement(i);
		delete info.url;
		}
	delete itsLinks;
	
	delete itsRegex;
	delete itsRegex2;
	delete itsRegex3;
}

/******************************************************************************
 GetLinkCount (protected)

 ******************************************************************************/

JSize
GMTextLink::GetLinkCount()
	const
{
	return itsLinks->GetElementCount();
}

/******************************************************************************
 GetLinkRange (protected)

 ******************************************************************************/

JIndexRange
GMTextLink::GetLinkRange
	(
	const JIndex index
	)
	const
{
	return (itsLinks->GetElement(index)).range;
}

/******************************************************************************
 LinkClicked (protected)

 ******************************************************************************/

void
GMTextLink::LinkClicked
	(
	const JIndex index
	)
{
	const LinkInfo info = itsLinks->GetElement(index);
	JString url = *(info.url);
	if (info.type == kEMail)
		{
		url.Prepend("mailto:");
		}
	else if (info.type == kWeb)
		{
		url.Prepend("http://");
		}
	(JXGetHelpManager())->ShowURL(url, NULL);
}

/******************************************************************************
 Receive (protected)

 ******************************************************************************/

void
GMTextLink::Receive
	(
	JBroadcaster* 	sender, 
	const Message& 	message
	)
{
	if (sender == itsEditor && message.Is(JTextEditor::kTextSet))
		{
		JSize linkCount = itsLinks->GetElementCount();
		for (JSize i = 1; i <= linkCount; i++)
			{
			LinkInfo info = itsLinks->GetElement(i);
			delete info.url;
			}
		itsLinks->RemoveAll();
		if (itsEditor->GetTextLength() > 0)
			{
			itsEditor->SetCaretLocation(1);
			JBoolean wrapped;
			JArray<JIndexRange> submatchList;
			while (itsEditor->SearchForward(*itsRegex, kFalse, 
											kFalse, &wrapped, &submatchList))
				{
				JIndexRange range = submatchList.GetElement(1);
				JString* str = new JString(itsEditor->GetText().GetSubstring(range));
				assert(str != NULL);
				LinkInfo info(range, str, kFullURL);
				itsLinks->AppendElement(info);
				}
			itsEditor->SetCaretLocation(1);
			while (itsEditor->SearchForward(*itsRegex2, kFalse, 
											kFalse, &wrapped, &submatchList))
				{
				JIndexRange range = submatchList.GetElement(1);
				JString* str = new JString(itsEditor->GetText().GetSubstring(range));
				assert(str != NULL);
				LinkInfo info(range, str, kEMail);
				itsLinks->AppendElement(info);
				}
			itsEditor->SetCaretLocation(1);
			while (itsEditor->SearchForward(*itsRegex3, kFalse, 
											kFalse, &wrapped, &submatchList))
				{
				JIndexRange range = submatchList.GetElement(1);
				JString* str = new JString(itsEditor->GetText().GetSubstring(range));
				assert(str != NULL);
				LinkInfo info(range, str, kWeb);
				itsLinks->AppendElement(info);
				}
			itsEditor->SetCaretLocation(1);
			}
		const JColorIndex blueColor = JGetCurrColormap()->GetBlueColor();

		linkCount = itsLinks->GetElementCount();
		for (JSize i = 1; i <= linkCount; i++)
			{
			const LinkInfo info = itsLinks->GetElement(i);
			itsEditor->SetSelection(info.range);
			itsEditor->SetCurrentFontColor(blueColor);
			itsEditor->SetCurrentFontUnderline(1);
			}
		if (itsEditor->GetTextLength() > 0)
			{
			itsEditor->SetCaretLocation(1);
			}
		}
}


#define JTemplateType GMTextLink::LinkInfo
#include <JArray.tmpls>
#undef JTemplateType
